"use strict";

exports.__esModule = true;
exports.default = void 0;

var _scrollIntoViewIfNeeded = _interopRequireDefault(require("scroll-into-view-if-needed"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var memoizedNow;

var now = function now() {
  if (!memoizedNow) {
    memoizedNow = 'performance' in window ? performance.now.bind(performance) : Date.now;
  }

  return memoizedNow();
};

var SCROLL_TIME = 300;

function ease(k) {
  return 0.5 * (1 - Math.cos(Math.PI * k));
}

function step(context) {
  var time = now();
  var value;
  var currentX;
  var currentY;
  var elapsed = (time - context.startTime) / SCROLL_TIME;
  elapsed = elapsed > 1 ? 1 : elapsed;
  value = ease(elapsed);
  currentX = context.startX + (context.x - context.startX) * value;
  currentY = context.startY + (context.y - context.startY) * value;
  context.method.call(context.scrollable, currentX, currentY);

  if (currentX !== context.x || currentY !== context.y) {
    requestAnimationFrame(step.bind(global, context));
  }
}

function smoothScroll(el, x, y, cb) {
  var scrollable;
  var startX;
  var startY;
  var method;
  var startTime = now();

  if (el === document.body || el === document.documentElement && true) {
    scrollable = window;
    startX = window.scrollX || window.pageXOffset;
    startY = window.scrollY || window.pageYOffset;
    method = window.scroll;
  } else {
    scrollable = el;
    startX = el.scrollLeft;
    startY = el.scrollTop;

    method = function method(x, y) {
      el.scrollLeft = x;
      el.scrollTop = y;
    };
  }

  step({
    scrollable: scrollable,
    method: method,
    startTime: startTime,
    startX: startX,
    startY: startY,
    x: x,
    y: y,
    cb: cb
  });
}

var _default = function _default(target, options) {
  if (options === void 0) {
    options = {};
  }

  var _options = options,
      _options$behavior = _options.behavior,
      behavior = _options$behavior === void 0 ? 'smooth' : _options$behavior;

  if (behavior !== 'smooth') {
    return (0, _scrollIntoViewIfNeeded.default)(target, options);
  }

  return (0, _scrollIntoViewIfNeeded.default)(target, _extends({}, options, {
    behavior: function behavior(instructions) {
      return Promise.all(instructions.map(function (_ref) {
        var el = _ref.el,
            left = _ref.left,
            top = _ref.top;
        return new Promise(function (resolve) {
          smoothScroll(el, left, top, function () {
            return resolve();
          });
        });
      }));
    }
  }));
};

exports.default = _default;
module.exports = exports["default"];