(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global.scrollIntoView = factory());
}(this, (function () { 'use strict';

  function _extends() {
    _extends = Object.assign || function (target) {
      for (var i = 1; i < arguments.length; i++) {
        var source = arguments[i];

        for (var key in source) {
          if (Object.prototype.hasOwnProperty.call(source, key)) {
            target[key] = source[key];
          }
        }
      }

      return target;
    };

    return _extends.apply(this, arguments);
  }

  function _extends$1() { _extends$1 = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends$1.apply(this, arguments); }

  // Compute what scrolling needs to be done on required scrolling boxes for target to be in view
  // The type names here are named after the spec to make it easier to find more information around what they mean:
  // To reduce churn and reduce things that need be maintained things from the official TS DOM library is used here
  // https://drafts.csswg.org/cssom-view/
  // For a definition on what is "block flow direction" exactly, check this: https://drafts.csswg.org/css-writing-modes-4/#block-flow-direction
  var isElement = function isElement(el) {
    return el != null && typeof el == 'object' && el.nodeType === 1;
  };

  function hasScrollableSpace(el, axis) {
    if (axis === 'Y') {
      return el.clientHeight < el.scrollHeight;
    }

    if (axis === 'X') {
      return el.clientWidth < el.scrollWidth;
    }

    return false;
  }

  function isScrollable(el) {
    var isScrollableY = hasScrollableSpace(el, 'Y');
    var isScrollableX = hasScrollableSpace(el, 'X');
    return isScrollableY || isScrollableX;
  }
  /**
   * Find out which edge to align against when logical scroll position is "nearest"
   * Interesting fact: "nearest" works similarily to "if-needed", if the element is fully visible it will not scroll it
   *
   * Legends:
   * ┌────────┐ ┏ ━ ━ ━ ┓
   * │ target │   frame
   * └────────┘ ┗ ━ ━ ━ ┛
   */


  var alignNearestBlock = function alignNearestBlock(targetStart, targetSize, frame, frameRect) {
    // targetSize is either targetRect.height or targetRect.width depending on if it's `block` or `inline`
    var targetEnd = targetStart + targetSize;
    var elementEdgeA = frame.scrollTop + targetStart;
    var elementEdgeB = frame.scrollTop + targetEnd;
    var scrollingEdgeA = frame.scrollTop;
    var height = frame === document.documentElement ? frame.clientHeight : frameRect.bottom;
    var scrollingEdgeB = document.documentElement ? frame.scrollTop + height : frameRect.bottom; //console.log('new coordinates', {elementEdgeA, elementEdgeB, scrollingEdgeA, scrollingEdgeB, targetStart, targetSize, 'frame.scrollTop': frame.scrollTop, 'frame.clientHeight': frame.clientHeight, 'frameRect.top': frameRect.top})

    /**
     * If element edge A and element edge B are both outside scrolling box edge A and scrolling box edge B
     *
     *          ┌──┐
     *        ┏━│━━│━┓
     *          │  │
     *        ┃ │  │ ┃        do nothing
     *          │  │
     *        ┗━│━━│━┛
     *          └──┘
     */

    if (elementEdgeA < scrollingEdgeA && elementEdgeB > scrollingEdgeB || elementEdgeA > scrollingEdgeA && elementEdgeB < scrollingEdgeB) {
      return 0;
    }
    /**
     * If element edge A is outside scrolling box edge A and element height is less than scrolling box height
     *
     *          ┌──┐
     *        ┏━│━━│━┓         ┏━┌━━┐━┓
     *          └──┘             │  │
     *  from  ┃      ┃     to  ┃ └──┘ ┃
     *
     *        ┗━ ━━ ━┛         ┗━ ━━ ━┛
     *
     */


    if (elementEdgeA < scrollingEdgeA && targetSize < height) {
      return targetStart;
    }
    /**
     * If element edge B is outside scrolling box edge B and element height is greater than scrolling box height
     *
     *        ┏━ ━━ ━┓         ┏━┌━━┐━┓
     *                           │  │
     *  from  ┃ ┌──┐ ┃     to  ┃ │  │ ┃
     *          │  │             │  │
     *        ┗━│━━│━┛         ┗━│━━│━┛
     *          │  │             └──┘
     *          │  │
     *          └──┘
     */


    if (elementEdgeB > scrollingEdgeB && targetSize > height) {
      return targetStart;
    }
    /**
     * If element edge B is outside scrolling box edge B and element height is less than scrolling box height
     *
     *        ┏━ ━━ ━┓         ┏━ ━━ ━┓
     *
     *  from  ┃      ┃     to  ┃ ┌──┐ ┃
     *          ┌──┐             │  │
     *        ┗━│━━│━┛         ┗━└━━┘━┛
     *          └──┘
     */
    ///*


    if (elementEdgeB > scrollingEdgeB && targetSize < height) {
      return elementEdgeB - scrollingEdgeB;
    } //*/

    /**
     * If element edge A is outside scrolling box edge A and element height is greater than scrolling box height
     *
     *          ┌──┐
     *          │  │
     *          │  │             ┌──┐
     *        ┏━│━━│━┓         ┏━│━━│━┓
     *          │  │             │  │
     *  from  ┃ └──┘ ┃     to  ┃ │  │ ┃
     *                           │  │
     *        ┗━ ━━ ━┛         ┗━└━━┘━┛
     */


    if (elementEdgeA < scrollingEdgeA && targetSize > height) {
      return elementEdgeB - scrollingEdgeB;
    }

    return 0;
  }; // Find out the


  var alignNearestInline = function alignNearestInline(targetStart, targetSize, frame, frameRect) {
    // targetSize is either targetRect.height or targetRect.width depending on if it's `block` or `inline`
    var targetEnd = targetStart + targetSize;
    var elementEdgeC = frame.scrollLeft + targetStart;
    var elementEdgeD = frame.scrollLeft + targetEnd;
    var scrollingEdgeC = frame.scrollLeft;
    var scrollingEdgeD = frame.scrollLeft + frame.clientWidth;
    console.log('test');
    /**
     *  If element edge C and element edge D are both outside scrolling box edge C and scrolling box edge D
     *
     *    ┏ ━ ━ ━ ━ ┓
     *   ┌───────────┐
     *   │┃         ┃│        Do nothing
     *   └───────────┘
     *    ┗ ━ ━ ━ ━ ┛
     */
    // @TODO

    /**
     * If element edge C is outside scrolling box edge C and element width is less than scrolling box width
     *
     *       from                 to
     *    ┏ ━ ━ ━ ━ ┓         ┏ ━ ━ ━ ━ ┓
     *  ┌───┐                 ┌───┐
     *  │ ┃ │       ┃         ┃   │     ┃
     *  └───┘                 └───┘
     *    ┗ ━ ━ ━ ━ ┛         ┗ ━ ━ ━ ━ ┛
     *
     * If element edge D is outside scrolling box edge D and element width is greater than scrolling box width
     *
     *       from                 to
     *    ┏ ━ ━ ━ ━ ┓         ┏ ━ ━ ━ ━ ┓
     *        ┌───────────┐   ┌───────────┐
     *    ┃   │     ┃     │   ┃         ┃ │
     *        └───────────┘   └───────────┘
     *    ┗ ━ ━ ━ ━ ┛         ┗ ━ ━ ━ ━ ┛
     */

    if (elementEdgeC < scrollingEdgeC && targetSize < frame.clientWidth || frameRect.left < 0 && targetEnd > frameRect.left && targetSize < frame.clientWidth) {
      return targetStart;
    }
    /**
     * If element edge C is outside scrolling box edge C and element width is greater than scrolling box width
     *
     *           from                 to
     *        ┏ ━ ━ ━ ━ ┓         ┏ ━ ━ ━ ━ ┓
     *  ┌───────────┐           ┌───────────┐
     *  │     ┃     │   ┃       │ ┃         ┃
     *  └───────────┘           └───────────┘
     *        ┗ ━ ━ ━ ━ ┛         ┗ ━ ━ ━ ━ ┛
     *
     * If element edge D is outside scrolling box edge D and element width is less than scrolling box width
     *
     *           from                 to
     *        ┏ ━ ━ ━ ━ ┓         ┏ ━ ━ ━ ━ ┓
     *                ┌───┐             ┌───┐
     *        ┃       │ ┃ │       ┃     │   ┃
     *                └───┘             └───┘
     *        ┗ ━ ━ ━ ━ ┛         ┗ ━ ━ ━ ━ ┛
     */


    if (elementEdgeD < scrollingEdgeD && targetSize < frame.clientWidth || frameRect.right > 0 && targetEnd > frameRect.right && targetSize < frame.clientWidth) {
      return targetEnd - frameRect.left - frame.clientWidth;
    }

    return 0;
  };

  var compute = function compute(target, options) {
    if (options === void 0) {
      options = {};
    }

    var _scrollMode$block$inl = _extends$1({
      scrollMode: 'always',
      block: 'center',
      inline: 'nearest'
    }, options),
        scrollMode = _scrollMode$block$inl.scrollMode,
        block = _scrollMode$block$inl.block,
        inline = _scrollMode$block$inl.inline,
        boundary = _scrollMode$block$inl.boundary;

    if (!isElement(target)) {
      throw new Error('Element is required in scrollIntoViewIfNeeded');
    }

    var targetRect = target.getBoundingClientRect(); // Collect all the scrolling boxes, as defined in the spec: https://drafts.csswg.org/cssom-view/#scrolling-box

    var frames = [];
    var parent;

    while (isElement(parent = target.parentNode) && target !== boundary) {
      if (isScrollable(parent)) {
        frames.push(parent);
      } // next tick


      target = parent;
    } // If the element is already visible we can end it here


    if (scrollMode === 'if-needed') {
      // @TODO optimize, as getBoundingClientRect is also called from computations loop
      var isVisible = frames.every(function (frame) {
        var frameRect = frame.getBoundingClientRect();

        if (targetRect.top < frameRect.top) {
          return false;
        }

        if (targetRect.bottom > frameRect.bottom) {
          return false;
        }

        if (frame === document.documentElement && targetRect.bottom > frame.clientHeight) {
          return false;
        }

        return true;
      });

      if (isVisible) {
        return [];
      }
    } // These values mutate as we loop through and generate scroll coordinates


    var targetBlock;
    var targetInline; // Collect new scroll positions

    var computations = frames.map(function (frame) {
      var frameRect = frame.getBoundingClientRect(); // @TODO fix hardcoding of block => top/Y

      var blockScroll = 0;
      var inlineScroll = 0; // @TODO fix the if else pyramid nightmare

      if (block === 'start') {
        if (!targetBlock) {
          targetBlock = targetRect.top;
        }

        if (document.documentElement === frame) {
          blockScroll = frame.scrollTop + targetBlock;
        } else {
          // prevent scrollTop values that overflow the scrollHeight
          var offset = Math.min(targetBlock - frameRect.top, frame.scrollHeight - frame.clientHeight - frame.scrollTop);
          blockScroll = frame.scrollTop + offset;
          targetBlock -= blockScroll - frame.scrollTop;
        }
      }

      if (block === 'center') {
        if (!targetBlock) {
          targetBlock = targetRect.top + targetRect.height / 2;
        }

        if (document.documentElement === frame) {
          blockScroll = frame.scrollTop + targetBlock - frame.clientHeight / 2;
        } else {
          // prevent negative scrollTop values
          var _offset = 0 - Math.min(frameRect.top + frameRect.height / 2 - targetBlock, frame.scrollTop);

          blockScroll = frame.scrollTop + _offset; // Cache the offset so that parent frames can scroll this into view correctly

          targetBlock += frame.scrollTop - blockScroll;
        }
      }

      if (block === 'end') {
        if (!targetBlock) {
          targetBlock = targetRect.bottom;
        }

        if (document.documentElement === frame) {
          blockScroll = frame.scrollTop + targetBlock - frame.clientHeight;
        } else {
          // prevent negative scrollTop values
          var _offset2 = 0 - Math.min(frameRect.bottom - targetBlock, frame.scrollTop);

          blockScroll = frame.scrollTop + _offset2; // Cache the offset so that parent frames can scroll this into view correctly

          targetBlock += frame.scrollTop - blockScroll;
        }
      }

      if (block === 'nearest') {
        if (!targetBlock) {
          targetBlock = targetRect.top;
        }

        var _offset3 = alignNearestBlock(targetBlock, targetRect.height, frame, frameRect);

        blockScroll = frame.scrollTop + _offset3;

        if (document.documentElement !== frame) {
          // Cache the offset so that parent frames can scroll this into view correctly
          targetBlock -= _offset3;
        }
      }

      if (inline === 'start') {
        if (!targetInline) {
          targetInline = targetRect.left;
        }

        if (document.documentElement === frame) {
          inlineScroll = frame.scrollLeft + targetInline;
        } else {
          // prevent scrollLeft values that overflow the scrollLeft
          var _offset4 = Math.min(targetInline - frameRect.left, frame.scrollLeft - frame.clientLeft - frame.scrollLeft);

          inlineScroll = frame.scrollLeft + _offset4;
          targetInline -= inlineScroll - frame.scrollLeft;
        }
      }

      if (inline === 'center') {
        if (!targetInline) {
          targetInline = targetRect.left + targetRect.width / 2;
        }

        if (document.documentElement === frame) {
          inlineScroll = frame.scrollLeft + targetInline - frame.clientWidth / 2;
        } else {
          // prevent negative scrollLeft values
          var _offset5 = 0 - Math.min(frameRect.left + frameRect.width / 2 - targetInline, frame.scrollLeft);

          inlineScroll = frame.scrollLeft + _offset5; // Cache the offset so that parent frames can scroll this into view correctly

          targetInline += frame.scrollLeft - inlineScroll;
        }
      }

      if (inline === 'end') {
        if (!targetInline) {
          targetInline = targetRect.right;
        }

        if (document.documentElement === frame) {
          inlineScroll = frame.scrollLeft + targetInline - frame.clientWidth;
        } else {
          // prevent negative scrollLeft values
          var _offset6 = 0 - Math.min(frameRect.right - targetInline, frame.scrollLeft);

          inlineScroll = frame.scrollLeft + _offset6; // Cache the offset so that parent frames can scroll this into view correctly

          targetInline += frame.scrollLeft - inlineScroll;
        }
      }

      if (inline === 'nearest') {
        if (!targetInline) {
          targetInline = targetRect.left;
        }

        var _offset7 = alignNearestInline(targetInline, targetRect.width, frame, frameRect);

        inlineScroll = frame.scrollLeft + _offset7;

        if (document.documentElement !== frame) {
          // Cache the offset so that parent frames can scroll this into view correctly
          targetInline -= _offset7;
        }
      }

      return {
        el: frame,
        top: blockScroll,
        left: inlineScroll
      };
    });
    return computations;
  };

  function _objectWithoutProperties$1(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

  function _extends$2() { _extends$2 = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends$2.apply(this, arguments); }
  // Some people might use both "auto" and "ponyfill" modes in the same file, so we also provide a named export so
  // that imports in userland code (like if they use native smooth scrolling on some browsers, and the ponyfill for everything else)
  // the named export allows this `import {auto as autoScrollIntoView, ponyfill as smoothScrollIntoView} from ...`
  var scrollIntoView = (function (target, maybeOptions) {
    if (maybeOptions === void 0) {
      maybeOptions = true;
    }

    var options = {}; // Handle alignToTop for legacy reasons

    if (maybeOptions === true || maybeOptions === null) {
      options = {
        block: 'start',
        inline: 'nearest'
      };
    } else if (maybeOptions === false) {
      options = {
        block: 'end',
        inline: 'nearest'
      };
    } else if (maybeOptions === Object(maybeOptions)) {
      // @TODO check if passing an empty object is handled like defined by the spec (for now it makes the web platform tests pass)
      options = Object.keys(maybeOptions).length === 0 ? {
        block: 'start',
        inline: 'nearest'
      } : _extends$2({
        block: 'center',
        inline: 'nearest'
      }, maybeOptions);
    }

    var _options = options,
        _options$behavior = _options.behavior,
        behavior = _options$behavior === void 0 ? 'auto' : _options$behavior,
        computeOptions = _objectWithoutProperties$1(_options, ["behavior"]); //return target.scrollIntoView(options)


    var instructions = compute(target, computeOptions);

    if (typeof behavior == 'function') {
      return behavior(instructions);
    }

    instructions.forEach(function (_ref) {
      var el = _ref.el,
          top = _ref.top,
          left = _ref.left;

      // browser implements the new Element.prototype.scroll API that supports `behavior`
      if (el.scroll) {
        el.scroll({
          top: top,
          left: left,
          behavior: behavior
        });
      } else {
        el.scrollTop = top;
        el.scrollLeft = left;
      }
    });
  });

  var memoizedNow;

  var now = function now() {
    if (!memoizedNow) {
      memoizedNow = 'performance' in window ? performance.now.bind(performance) : Date.now;
    }

    return memoizedNow();
  };

  var SCROLL_TIME = 300;

  function ease(k) {
    return 0.5 * (1 - Math.cos(Math.PI * k));
  }

  function step(context) {
    var time = now();
    var value;
    var currentX;
    var currentY;
    var elapsed = (time - context.startTime) / SCROLL_TIME;
    elapsed = elapsed > 1 ? 1 : elapsed;
    value = ease(elapsed);
    currentX = context.startX + (context.x - context.startX) * value;
    currentY = context.startY + (context.y - context.startY) * value;
    context.method.call(context.scrollable, currentX, currentY);

    if (currentX !== context.x || currentY !== context.y) {
      requestAnimationFrame(step.bind(global, context));
    }
  }

  function smoothScroll(el, x, y, cb) {
    var scrollable;
    var startX;
    var startY;
    var method;
    var startTime = now();

    if (el === document.body || el === document.documentElement && true) {
      scrollable = window;
      startX = window.scrollX || window.pageXOffset;
      startY = window.scrollY || window.pageYOffset;
      method = window.scroll;
    } else {
      scrollable = el;
      startX = el.scrollLeft;
      startY = el.scrollTop;

      method = function method(x, y) {
        el.scrollLeft = x;
        el.scrollTop = y;
      };
    }

    step({
      scrollable: scrollable,
      method: method,
      startTime: startTime,
      startX: startX,
      startY: startY,
      x: x,
      y: y,
      cb: cb
    });
  }

  var index = (function (target, options) {
    if (options === void 0) {
      options = {};
    }

    var _options = options,
        _options$behavior = _options.behavior,
        behavior = _options$behavior === void 0 ? 'smooth' : _options$behavior;

    if (behavior !== 'smooth') {
      return scrollIntoView(target, options);
    }

    return scrollIntoView(target, _extends({}, options, {
      behavior: function behavior(instructions) {
        return Promise.all(instructions.map(function (_ref) {
          var el = _ref.el,
              left = _ref.left,
              top = _ref.top;
          return new Promise(function (resolve) {
            smoothScroll(el, left, top, function () {
              return resolve();
            });
          });
        }));
      }
    }));
  });

  return index;

})));
